from typing import Optional

from orchestrator.domain.base import ProductBlockModel
from orchestrator.types import SubscriptionLifecycle
import ipaddress


class DeviceBlockInactive(ProductBlockModel,
                          lifecycle=[SubscriptionLifecycle.INITIAL],
                          product_block_name="DeviceBlock"):
    fqdn: Optional[str] = None
    ts_address: Optional[str] = None
    ts_port: Optional[int] = None
    lo_ipv4_address: Optional[ipaddress.IPv4Address] = None
    lo_ipv6_address: Optional[ipaddress.IPv6Address] = None
    lo_iso_address: Optional[str] = None
    si_ipv4_network: Optional[ipaddress.IPv4Network] = None
    ias_lt_ipv4_network: Optional[ipaddress.IPv4Network] = None
    ias_lt_ipv6_network: Optional[ipaddress.IPv6Network] = None
    site_city: Optional[str] = None
    site_country: Optional[str] = None
    site_country_code: Optional[str] = None
    site_latitude: Optional[str] = None
    site_longitude: Optional[str] = None
    snmp_location: Optional[str] = None


class DeviceBlockProvisioning(DeviceBlockInactive,
                              lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    fqdn: str
    ts_address: str
    ts_port: str
    lo_ipv4_address: Optional[ipaddress.IPv4Address] = None
    lo_ipv6_address: Optional[ipaddress.IPv6Address] = None
    lo_iso_address: Optional[str] = None
    si_ipv4_network: Optional[ipaddress.IPv4Network] = None
    ias_lt_ipv4_network: Optional[ipaddress.IPv4Network] = None
    ias_lt_ipv6_network: Optional[ipaddress.IPv6Network] = None
    site_city: Optional[str] = None
    site_country: Optional[str] = None
    site_country_code: Optional[str] = None
    site_latitude: Optional[str] = None
    site_longitude: Optional[str] = None
    snmp_location: Optional[str] = None


class DeviceBlock(DeviceBlockProvisioning,
                  lifecycle=[SubscriptionLifecycle.ACTIVE]):
    fqdn: str
    ts_address: str
    ts_port: str
    lo_ipv4_address: ipaddress.IPv4Address
    lo_ipv6_address: ipaddress.IPv6Address
    lo_iso_address: str
    si_ipv4_network: ipaddress.IPv4Network
    ias_lt_ipv4_network: ipaddress.IPv4Network
    ias_lt_ipv6_network: ipaddress.IPv6Network
    site_city: str
    site_country: str
    site_country_code: str
    site_latitude: str
    site_longitude: str
    snmp_location: str
