from typing import Optional

from orchestrator.domain.base import ProductBlockModel
from orchestrator.types import SubscriptionLifecycle
import ipaddress

#  - id: GS-00012
#    name: AMS2-LON-IPTRUNK-100G
#    config:
#      common:
#        speed: '100'
#        is_leased_line: false
#        isis_metric: '9000'
#        minimum_links: 1
#      nodeA:
#        name: rt2.ams.nl.geant.net
#        ae_name: ae2
#        ipv4_address: 62.40.98.164/31
#        ipv6_address: 2001:798:cc::5/126
#        members:
#          - et-0/0/0
#        port_id: GA-02036
#      nodeB:
#        name: mx1.lon.uk.geant.net
#        ae_name: ae8
#        ipv4_address: 62.40.98.165/31
#        ipv6_address: 2001:798:cc::6/126
#        members:
#          - et-4/0/2
#        port_id: GA-01834


class IptrunkBlockInactive(ProductBlockModel,
                          lifecycle=[SubscriptionLifecycle.INITIAL],
                          product_block_name="IptrunkBlock"):
    geant_s_sid: Optional[str] = None
    iptrunk_speed: Optional[str] = None
    iptrunk_minimum_links: Optional[int] = None
    iptrunk_ipv4_network: Optional[ipaddress.IPv4Network] = None
    iptrunk_ipv6_network: Optional[ipaddress.IPv6Network] = None
    #
    iptrunk_sideA_fqdn: Optional[str] = None
    iptrunk_sideA_ae_iface: Optional[str] = None
    iptrunk_sideA_ae_geant_a_sid: Optional[str] = None
    iptrunk_sideA_ae_members: Optional[list] = None
    #
    iptrunk_sideB_fqdn: Optional[str] = None
    iptrunk_sideB_ae_iface: Optional[str] = None
    iptrunk_sideB_ae_geant_a_sid: Optional[str] = None
    iptrunk_sideB_ae_members: Optional[list] = None


class IptrunkBlockProvisioning(IptrunkBlockInactive,
                              lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    geant_s_sid: Optional[str] = None
    iptrunk_speed: Optional[str] = None
    iptrunk_minimum_links: Optional[int] = None
    iptrunk_ipv4_network: Optional[ipaddress.IPv4Network] = None
    iptrunk_ipv6_network: Optional[ipaddress.IPv6Network] = None
    #
    iptrunk_sideA_fqdn: Optional[str] = None
    iptrunk_sideA_ae_iface: Optional[str] = None
    iptrunk_sideA_ae_geant_a_sid: Optional[str] = None
    iptrunk_sideA_ae_members: Optional[list] = None
    #
    iptrunk_sideB_fqdn: Optional[str] = None
    iptrunk_sideB_ae_iface: Optional[str] = None
    iptrunk_sideB_ae_geant_a_sid: Optional[str] = None
    iptrunk_sideB_ae_members: Optional[list] = None


class IptrunkBlock(IptrunkBlockProvisioning,
                  lifecycle=[SubscriptionLifecycle.ACTIVE]):
    geant_s_sid: str
    iptrunk_speed: str
    iptrunk_minimum_links: int
    iptrunk_ipv4_network: ipaddress.IPv4Network
    iptrunk_ipv6_network: ipaddress.IPv6Network
    #
    iptrunk_sideA_fqdn: str
    iptrunk_sideA_ae_iface: str
    iptrunk_sideA_ae_geant_a_sid: str
    iptrunk_sideA_ae_members: list
    #
    iptrunk_sideB_fqdn: str
    iptrunk_sideB_ae_iface: str
    iptrunk_sideB_ae_geant_a_sid: str
    iptrunk_sideB_ae_members: list
